;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                                                                               ;;
;;  Behavioral Spear Phishing Risk Simulation Tool, by Kim Kaivanto, November 2013                                               ;;
;;  Simulates 100 agents, whose responses to spear phishing emails are calculated, illustrated and recorded.                     ;;
;;  The vertical height of the world represents the duration of the spear-phishing attack in weeks.                              ;;
;;                                                                                                                               ;;
;;  Agents may be M0s (classical Signal Detection Theory (SDT), i.e. normatively rational, risk-neutral decision makers          ;;
;;                M1s ('behavioral' CPT-SDT decision makers)                                                                     ;;
;;                M2s ('behavioral' decision makers who employ CPT-SDT *and* are vulnerable to peripheral-route persuasion       ;;
;;                                                                                                                               ;;
;;  Requires the 'R-Extension' package available here http://r-ext.sourceforge.net/                                              ;;
;;                                                                                                                               ;;
;;  Height of world indicates duration, in weeks, of the spear-phishing attack (default 3 rows = 3 weeks                         ;;
;;                                                                                                                               ;;
;;  Once max-iter is reached and the run is complete, type the following in the command center: show phished-per-iter-list       ;;
;;  The resulting vector may be highlighted, copied and saved as a text file for analysis in a statistical software package      ;;
;;                                                                                                                               ;;
;;  Code licenced by Kim Kaivanto, http://www.lums.lancs.ac.uk/profiles/kim-kaivanto/                                            ;;
;;  under a Creative Commons Attribtion-Noncommercial-Share Alike 3.0                                                            ;;
;;  Unsupported License (see http://creativecommons.org/licenses/by-nc-sa/3.0/                                                   ;;
;;  If this model is used in original or modified form for research, please cite                                                 ;;
;;  (i) the code source on modelingcommons, as well as                                                                           ;;
;;  (ii) the published paper:                                                                                                    ;;
;;       Kaivanto K (2014) "The Effect of Decentralized Behavioral Decision Making on System-Level Risk",                        ;;
;;       Risk Analysis 34(12), pp. 2121--2142. DOI: 10.1111/risa.12219                                                           ;;
;;       available at the URL http://onlinelibrary.wiley.com/doi/10.1111/risa.12219/                                             ;;
;;                                                                                                                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


extensions [r] 

globals [
  iter                           ;; iteration count tracker
  phi                            ;; CPT value function exponent
  prob-of-phish                  ;; prior probability that an email will be a spear-phishing email
  phished-per-iter-list          ;; a list variable of length max-iter that records the number of security breaches (total) within each iteration
  ]
;; controlled by slider:
;;    prob-periph-pers              probability that peripherral-route persuasion is 'successful', causing the individual to use his/her 'lower' ROC curve
;;    no-of-spearphish-per-week     how many, out of the 250 emails per week, are spear phishing emails
;;    d-prime-vigilant              d' of the normal ROC curve
;;    d-prime-low                   d' when compromised by peripher-route persuasion (see prob-periph-pers above)
;;    max-iter                      maximum number of iterations to run, ranges from 1 to 10,000 
;;
;; controlled by drop-down menu:
;;    class-of-users                mzeros for M0s, mones for M1s, mtwos for M2s
;;                                  for detailed explanation of user classes see:
;;                                  Kaivanto (2013) "The Effect of Decentralized Behavioral Decision Making on System-Level Risk"

breed [ mzeros mzero ]  ;; Benchmark, normative rationality
breed [ mones mone ]    ;; CPT-SDT
breed [ mtwos mtwo ]    ;; CPT-SDT + psychology of deception (peripheral-route persuasion, visceral emotion, time pressure, contextual cues)

turtles-own [
  d-prime
  phished?      ;; in the current iteration, has the agent clicked on a phishing email? 
  theta-opt     ;; optimal cutoff threshold under the classical SDT (mzeros) or under the CPT-SDT (mones and mtwos)
  alpha         ;; false positive likelihood associated with the agent's theta-opt
  power         ;; true positive likelihood associated with the agent's theta-opt
  ]


;;           SETUP
to setup
  clear-all
  ask patches [ set pcolor black ]
  set iter 0
  set phished-per-iter-list (list)
  set prob-of-phish ( no-of-spearphish-per-week / 250 )
  if ( class-of-users = "mzeros" ) 
    [ create-mzeros 100 
        [ 
          defaults 
          calculate-mzeros-theta-opt
          set phished? false
        ] 
    ]
  if ( class-of-users = "mones" ) 
    [ create-mones 100  
        [ 
          defaults 
          calculate-mones-mtwos-theta-opt
          set phished? false
        ] 
    ]
  if ( class-of-users = "mtwos" ) 
    [create-mtwos 100 
      [ defaults 
        ifelse (( random 100) < prob-periph-pers ) 
          [ set d-prime d-prime-low ] [ set d-prime d-prime-vigilant ]
        calculate-mones-mtwos-theta-opt
        set phished? false
      ] 
    ]
  reset-ticks
end

;;            defaults for use in SETUP of agents 
to defaults
    set color gray
    set heading 0
    set xcor (who) 
    set ycor min-pycor 
    set phi 0.88
    set d-prime d-prime-vigilant   ;; the default value for mzeros and mones, and the starting default for mtwos
end


to calculate-mzeros-theta-opt
      set theta-opt ((1 / d-prime)*(ln C-FP - ln (C-FN - C-TP) + ln (1 - prob-of-phish ) - ln prob-of-phish + (((d-prime)^(2)) / 2))) 
end

to calculate-mones-mtwos-theta-opt
      set theta-opt ((1 / d-prime)*((phi)*(ln (C-FP)) - ln (((C-FN)^(phi)) - ((C-TP)^(phi))) + ln ( 1 - prob-of-phish ) - ln prob-of-phish + (((d-prime)^(2)) / 2)))
end

;;
to go
  ask-concurrent turtles [ sdt ] ;; 
;; 
  if ( all? turtles [ ycor = max-pycor ] ) and ( iter = max-iter )
    [ tick
      set phished-per-iter-list lput ( count turtles with [ phished? ] ) phished-per-iter-list
      if  pause?  [ user-message (word "End of iteration #" iter ". End of run.") ]
      histogram-plot
      user-message (word "Finished!")      ;;;;;;;;;
      stop ]
;;
  if ( all? turtles [ ycor = max-pycor ] ) and ( iter < max-iter )
    [ 
      tick
      if  pause?  [ user-message (word "End of iteration #" iter ".") ]
      ask patches [ set pcolor black ]
      ask-concurrent turtles [ set ycor min-pycor ]
      set phished-per-iter-list lput ( count turtles with [ phished? ] ) phished-per-iter-list
      ask-concurrent turtles [ set phished? false ]
      ask-concurrent turtles [ sdt ] ;; 
    ]  
;;
  if (all? turtles [ ycor = max-pycor - 1 ])
    [
      tick
      ask-concurrent turtles [ forward 1 ]
      set iter iter + 1
    ]  
;;
  if ( all? turtles [ ycor < max-pycor - 1])
    [ 
      tick
      ask-concurrent turtles [ forward 1 ]
    ]
end



to sdt
  ifelse phished? 
  [
    set pcolor lime
  ]
  [
    r:put "thetastar"  theta-opt 
    r:put "dprime" d-prime
    r:eval "power <- pnorm( dprime - thetastar )"
    r:eval "alpha <- pnorm( 0 - thetastar )"
    set alpha (r:get "alpha") 
    set power (r:get "power")
    ifelse ((random-float 1 ) <= power) 
    [ set phished? false 
      set pcolor lime
    ] 
    [ set phished? true 
      set pcolor red
    ]
  ]   
  
end

to histogram-plot
  r:setPlotDevice
  r:put "phishperiter" phished-per-iter-list
  r:eval "hist(phishperiter, xlim=c(0,80), ylim=c(0,50),breaks=16)"
end
@#$#@#$#@
GRAPHICS-WINDOW
235
10
1045
65
-1
-1
8.0
1
10
1
1
1
0
0
0
1
0
99
0
2
0
0
1
ticks
60.0

SLIDER
4
70
176
103
d-prime-vigilant
d-prime-vigilant
0
5
3
.25
1
NIL
HORIZONTAL

SLIDER
4
105
176
138
d-prime-low
d-prime-low
0
d-prime-vigilant
0.5
.25
1
NIL
HORIZONTAL

BUTTON
1056
13
1119
46
GO
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

BUTTON
1126
13
1190
46
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
1050
58
1222
91
max-iter
max-iter
1
10000
10000
100
1
NIL
HORIZONTAL

CHOOSER
4
11
142
56
class-of-users
class-of-users
"mzeros" "mones" "mtwos"
2

SWITCH
1051
96
1154
129
pause?
pause?
1
1
-1000

TEXTBOX
179
30
308
142
d'        AUC\n0.5     .638\n1.0     .760\n1.5     .856\n2.0     .921\n2.5     .961\n3.0     .983\n3.5     .993
11
0.0
1

SLIDER
3
223
175
256
prob-periph-pers
prob-periph-pers
0
100
5
1
1
%
HORIZONTAL

SLIDER
3
269
175
302
C-FP
C-FP
0
10
0.25
0.25
1
NIL
HORIZONTAL

SLIDER
2
305
174
338
C-TP
C-TP
C-FP
10
0.5
0.25
1
NIL
HORIZONTAL

SLIDER
3
341
175
374
C-FN
C-FN
C-TP
30
20
0.25
1
NIL
HORIZONTAL

SLIDER
4
160
196
193
no-of-spearphish-per-week
no-of-spearphish-per-week
1
10
5
1
1
/ 250
HORIZONTAL

TEXTBOX
7
148
204
170
Number of phishing emails per week (of 250)
9
0.0
1

TEXTBOX
7
200
184
222
Probability of peripheral-route persuasion displacing vigilance.
9
0.0
1

MONITOR
58
398
113
443
phished 
count turtles with [phished?]
0
1
11

MONITOR
4
397
54
442
iter no.
iter
17
1
11

MONITOR
120
398
198
443
ave per tick
(sum phished-per-iter-list) / ticks
3
1
11

PLOT
414
149
944
429
phished per iter
No phished per iter
frequency
0.0
100.0
0.0
50.0
true
false
"set-histogram-num-bars 20" ""
PENS
"default" 1.0 1 -2674135 true "" "histogram phished-per-iter-list"

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
